--[[ 

	Simple Mount Addon
	Binds "NUMPAD-1" temporarily to "Mount/Unmount"
	Martin Karer / Sezz, 2010-2011

	Changelog:
	2011-07-22:
		[*] Vashj'ir fixes (addes some zones)

	2011-07-09:
		[*] GetWintergraspWaitTime fix
		
	2011-06-05:
		[+] Winged Guardian

	2011-05-06:
		[+] Added some new mounts

	2010-12-25:
		[*] Vashj'ir support
		[+] Added rare Cataclysm mounts

	2010-11-25:
		[*] Fixed Flight Master's License

	2010-11-07:
		[+] Added Druid's Flight Form

	2010-11-08:
		[*] Fixed "first-run" bug

	2010-11-07:
		[!] Cast Druid's Flight Form while moving

	2010-11-04:
		[+] Separated from s:Burro
		[+] Added random RARE mount feature
		[!] Switch back to indexes to fix Big Blizzard Bear

	2010-09-12:
		[:] Cataclysm Update
		[+] Vashj'ir support (but not complete)

	2010-02-05:
		[+] Initial Release

--]]


local addonName = "sMount";
local addon = CreateFrame("Frame", addonName);
local mountTypes = {["1"] = "Ground", ["2"] = "Air" };

local vashjirZones = { "Kelp'thar Forest", "Shimmering Expanse", "Abyssal Depths", "Damplight Chamber", "Ruins of Vashj'ir" };
local nonFlyableAreas = { "Throne of the Four Winds" };

local GetZoneText = GetZoneText;
local GetSubZoneText = GetSubZoneText;


-- Rare Mounts
local rareMountsAvailable = {};
local rareMounts = {
	["1"] = {
		98204, -- Amani Battle Bear
		43688, -- Amani War Bear
		68188, -- Crusader's Black Warhorse
		68187, -- Crusader's White Warhorse
		17481, -- Deathcharger's Reins
		36702, -- Fiery Warhorse's Reins
		72286, -- Invincible's Reins
		73313, -- Reins of the Crimson Deathcharger
		61467, -- Reins of the Grand Black War Mammoth
		61465, -- Reins of the Grand Black War Mammoth
		41252, -- Reins of the Raven Lord
		68056, -- Swift Horde Wolf
		68057, -- Swift Alliance Steed
		46628, -- Swift White Hawkstrider
		24242, -- Swift Razzashi Raptor
		24252, -- Swift Zulian Tiger
		26656, -- Black Qiraji Resonating Crystal
		60424, -- Mekgineer's Chopper
--		55531, -- Mechano-hog
		58983, -- Big Blizzard Bear
		49322, -- Swift Zhevra
		74918, -- Wooly White Rhino
		51412, -- Big Battle Bear
		65917, -- Magic Rooster Egg
		42777, -- Reins of the Swift Spectral Tiger
		42776, -- Reins of the Spectral Tiger
		49379, -- Great Brewfest Kodo
--		43900, -- Swift Brewfest Ram
		43899, -- Brewfest Ram
		48025, -- The Horseman's Reins
--		39316, -- Reins of the Dark Riding Talbuk
--		34790, -- Reins of the Dark War Talbuk
		75614, -- Celestial Steed
		92155, -- Ultramarine Qiraji Battle Tank
--		88749, -- Tan Riding Camel
		92232, -- Spectral Wolf
		92231, -- Spectral Steed
--		93644, -- Kor'kron Annihilator
		88750, -- Grey Riding Camel
		90621, -- Golden King
--		84751, -- Fossilized Raptor
--		88748, -- Brown Riding Camel
		96499, -- Swift Zulian Panther
		96491, -- Armored Razzashi Raptor
		98727, -- Winged Guardian
		97359, -- Flameward Hippogryph
		},
	["2"] = {
		40192, -- Ashes of Al'ar
		72286, -- Invincible's Reins
		63796, -- Mimiron's Head
--		59650, -- Reins of the Black Drake
		59571, -- Reins of the Twilight Drake
		59568, -- Reins of the Blue Drake
		59567, -- Reins of the Azure Drake
		59996, -- Reins of the Blue Proto-Drake
		61294, -- Reins of the Green Proto-Drake
		61996, -- Blue Dragonhawk Mount
		61997, -- Red Dragonhawk Mount
		59976, -- Reins of the Black Proto-Drake
		63956, -- Reins of the Ironbound Proto-Drake
		60021, -- Reins of the Plagued Proto-Drake
--		59961, -- Reins of the Red Proto-Drake
		63963, -- Reins of the Rusted Proto-Drake
		60024, -- Reins of the Violet Proto-Drake
		46197, -- X-51 Nether-Rocket
		46199, -- X-51 Nether-Rocket X-TREME
		48025, -- The Horseman's Reins
		71342, -- Big Love Rocket
		75614, -- Celestial Steed
		72807, -- Reins of the Icebound Frostbrood Vanquisher
		72808, -- Reins of the Bloodbathed Frostbrood Vanquisher
		93326, -- Sandstone Drake
		88331, -- Volcanic Stone Drake
		88746, -- Vitreous Stone Drake
		88718, -- Phosphorescent Stone Drake
		88741, -- Drake of the West Wind
		88744, -- Drake of the South Wind
		88742, -- Drake of the North Wind
		88335, -- Drake of the East Wind
		88990, -- Dark Phoenix
		98727, -- Winged Guardian
		97359, -- Flameward Hippogryph
		},
	};


-- Basic Stuff
addon.DisableEvent = function(self, event, ...)
	self:UnregisterEvent(event);
	self[event] = nil;
end

addon:RegisterEvent("PLAYER_ENTERING_WORLD");
addon.PLAYER_ENTERING_WORLD = function(self, event)
	self.UPDATE_BINDINGS();
	addon:ScanRareMounts();
	self.DisableEvent(self, event);
end

addon:RegisterEvent("UPDATE_BINDINGS");
addon.UPDATE_BINDINGS = function()
	if (not InCombatLockdown()) then
		SetOverrideBindingClick(sMountButton, true, "NUMPAD1", sMountButton:GetName());
	end
end

addon:RegisterEvent("VARIABLES_LOADED");
addon.VARIABLES_LOADED = function()
	if (not sMounts) then
		sMounts = {};
	end
end


-- Little Helpers
local print = function(message)
	ChatFrame1:AddMessage("[|cffeb83cas:|rMount] "..tostring(message))
end

local GetSpellName = function(spellId)
	local spellName = GetSpellInfo(spellId);
	return spellName;
end

local GetRandomMount = function(mountType)
	local mountNum = #rareMountsAvailable[mountType];
	if (mountNum > 0) then
		local mountIndex = math.random(1, mountNum);

		return GetSpellName(rareMountsAvailable[mountType][mountIndex]);
	else
		print("ERROR: Get a rare mount first to use the random mount feature!")
		print("You can also setup specific mounts with this commands: ");
		print("GROUND: /setmount 1 [Link]");
		print("AIR: /setmount 2 [Link]");

		return nil;
	end
end


-- ShapeShifting
local _, playerClass = UnitClass("player");
local shapeShiftAction;

if (playerClass == "SHAMAN") then
	shapeShiftAction = "!"..GetSpellName(2645);
elseif (playerClass == "DRUID") then
	shapeShiftAction = "[flyable] "..GetSpellName(40120).."; [swimming] "..GetSpellName(1066).."; "..GetSpellName(783);
elseif (playerClass == "MAGE") then
	shapeShiftAction = GetSpellName(130);
end

if (shapeShiftAction) then
	addon.PLAYER_REGEN_DISABLED = function() addon:UpdateButton(shapeShiftAction); end
	addon:RegisterEvent("PLAYER_REGEN_DISABLED");
end


-- Enable Event Handler
addon:SetScript("OnEvent",function(self, event, ...) self[event](self, event, ...); end);


-- Magic
local IsFlyableArea = IsFlyableArea;
--local GetCurrentMapContinent = GetCurrentMapContinent;
--local IsLearnedFlyableArea = function()
--	local flyable = IsFlyableArea();
--	
--	if (flyable) then
--		local zone = GetCurrentMapContinent();
--		if (zone == 1 or zone == 2) then
--			flyable = IsSpellKnown(90269);
--		end
--	end
--	
--	return flyable;
--end

function addon:ScanRareMounts(mountType)
	if (mountType) then
		rareMountsAvailable[mountType] = {};
		for i = 1, GetNumCompanions("MOUNT"), 1 do
			local _, _, spellId = GetCompanionInfo("Mount", i);
			if (tContains(rareMounts[mountType], spellId)) then
				table.insert(rareMountsAvailable[mountType], spellId);
			end
		end
	else
		addon:ScanRareMounts("1");
		addon:ScanRareMounts("2");
		rareMounts = nil;
	end
end

function addon:UpdateButton(mountName)
	if (mountName) then
		sMountButton:SetAttribute("macrotext", "/use "..mountName);
	else
		sMountButton:SetAttribute("macrotext", nil);
	end
end

function addon:Mount()
	-- Unmount first!
	if (IsMounted()) then
		Dismount();
	elseif (CanExitVehicle()) then
		VehicleExit();
	end
	
	if (not InCombatLockdown()) then
		-- Out of combat!
		local zone = GetZoneText();
		local isMoving = (GetUnitSpeed("player") ~= 0);

		if (not IsOutdoors() or (isMoving and shapeShiftAction)) then
			-- Moving around...
			addon:UpdateButton(shapeShiftAction);
		elseif (IsSwimming() and tContains(vashjirZones, zone)) then
			-- Sea Horse
			addon:UpdateButton("Abyssal Seahorse");
		else
			-- P0sing!
			local canFly = (IsFlyableArea() and not tContains(nonFlyableAreas, zone)) or (zone == "Dalaran" and GetSubZoneText() == "Krasus' Landing") or (zone == "Wintergrasp" and GetWintergraspWaitTime and GetWintergraspWaitTime()) or false;
			local mountType = canFly and "2" or "1";
			
			if (sMounts[mountType] ~= nil) then
				addon:UpdateButton(sMounts[mountType]);
			else
				addon:UpdateButton(GetRandomMount(mountType));
			end
		end
	end
end


-- Set Mount Command
_G["SLASH_"..addonName.."1"] = "/setmount";
SlashCmdList[addonName] = function(cmd)
	local mountType, mountLink = cmd:match("^([^%s]+)%s*(.*)$");
	local mountName;
	
	if (mountLink) then
		_, _, _, _, mountId, _, _, _, _, _, _, _, _, mountName = string.find(mountLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
	end
	
	if (mountType == "1" or mountType == "2") then
		if (mountName) then
			print("["..mountTypes[mountType].."] "..mountName);
		else
			print("["..mountTypes[mountType].."] Random (RARE) Mount");
		end
		
		sMounts[mountType] = mountName;
	end
end
